import { connect } from 'react-redux'
import { actions } from 'pubsweet-client'
import { ConnectPage } from 'xpub-connect'
import { withJournal } from 'xpub-journal'
import { head, get, isEmpty } from 'lodash'
import { replace } from 'react-router-redux'
import { withRouter } from 'react-router-dom'
import {
  selectFragment,
  selectCollection,
  selectCurrentUser,
} from 'xpub-selectors'
import { get as apiGet } from 'pubsweet-client/src/helpers/api'
import {
  compose,
  withState,
  lifecycle,
  withProps,
  withHandlers,
  setDisplayName,
} from 'recompose'
import { getSignedUrl } from 'pubsweet-components-faraday/src/redux/files'
import { reviewerDecision } from 'pubsweet-components-faraday/src/redux/reviewers'
import {
  hasManuscriptFailure,
  clearCustomError,
} from 'pubsweet-components-faraday/src/redux/errors'
import { selectEditorialRecommendations } from 'pubsweet-components-faraday/src/redux/recommendations'
import {
  getUserToken,
  currentUserIs,
  canMakeRevision,
  canMakeDecision,
  canEditManuscript,
  currentUserIsReviewer,
  canMakeRecommendation,
  canOverrideTechnicalChecks,
} from 'pubsweet-component-faraday-selectors'

import ManuscriptLayout from './ManuscriptLayout'
import { parseSearchParams, redirectToError } from './utils'
import {
  canAssignHE,
  selectFetching,
  getHandlingEditors,
  assignHandlingEditor,
  revokeHandlingEditor,
  selectHandlingEditors,
} from '../redux/editors'

export default compose(
  setDisplayName('ManuscriptPage'),
  withJournal,
  withRouter,
  withState('editorInChief', 'setEiC', 'N/A'),
  ConnectPage(({ match }) => [
    actions.getCollection({ id: match.params.project }),
    actions.getFragments({ id: match.params.project }),
  ]),
  connect(
    (state, { match }) => ({
      currentUser: selectCurrentUser(state),
      handlingEditors: selectHandlingEditors(state),
      hasManuscriptFailure: hasManuscriptFailure(state),
      fragment: selectFragment(state, match.params.version),
      collection: selectCollection(state, match.params.project),
      editorialRecommendations: selectEditorialRecommendations(
        state,
        match.params.version,
      ),
    }),
    {
      replace,
      getSignedUrl,
      clearCustomError,
      reviewerDecision,
      assignHandlingEditor,
      revokeHandlingEditor,
      getFragment: actions.getFragment,
      getCollection: actions.getCollection,
      updateVersion: actions.updateFragment,
    },
  ),
  connect((state, { currentUser, match, collection, fragment }) => ({
    currentUser: {
      ...currentUser,
      token: getUserToken(state),
      isEIC: currentUserIs(state, 'adminEiC'),
      isReviewer: currentUserIsReviewer(state),
      canAssignHE: canAssignHE(state, match.params.project),
    },
    isFetching: {
      editorsFetching: selectFetching(state),
    },
    permissions: {
      canMakeRevision: canMakeRevision(state, collection, fragment),
      canMakeDecision: canMakeDecision(state, collection, fragment),
      canEditManuscript: canEditManuscript(state, collection, fragment),
      canOverrideTechChecks: canOverrideTechnicalChecks(state, collection),
      canMakeRecommendation: canMakeRecommendation(state, collection, fragment),
    },
  })),
  ConnectPage(({ currentUser, handlingEditors, collection }) => {
    if (currentUser.isEIC) {
      return [getHandlingEditors()]
    }
    return []
  }),
  withHandlers({
    updateManuscript: ({ updateVersion, collection, fragment }) => data =>
      updateVersion(collection, {
        id: fragment.id,
        ...data,
      }),
    setEditorInChief: ({ setEiC }) => eic => {
      if (eic) {
        const { firstName = '', lastName = '' } = eic
        setEiC(`${firstName} ${lastName}`)
      }
    },
    assignHE: ({
      fragment,
      collection,
      getFragment,
      getCollection,
      assignHandlingEditor,
      collection: { id: collectionId },
    }) => (email, modalProps) =>
      assignHandlingEditor({
        email,
        collectionId,
      })
        .then(() => {
          getCollection({ id: collectionId })
          getFragment(collection, fragment)
          modalProps.hideModal()
        })
        .catch(() => modalProps.setModalError('Oops! Something went wrong.')),
    revokeHE: ({
      getCollection,
      revokeHandlingEditor,
      collection: { id: collectionId },
    }) => (invitationId, modalProps) =>
      revokeHandlingEditor({
        invitationId,
        collectionId,
      })
        .then(() => {
          getCollection({ id: collectionId })
          modalProps.hideModal()
        })
        .catch(() => modalProps.setModalError('Oops! Something went wrong.')),
  }),
  lifecycle({
    componentDidMount() {
      const {
        match,
        replace,
        history,
        location,
        getFragment,
        getCollection,
        reviewerDecision,
        setEditorInChief,
        clearCustomError,
        hasManuscriptFailure,
      } = this.props
      if (hasManuscriptFailure) {
        history.push('/not-found')
        clearCustomError()
      }

      const collectionId = match.params.project
      const fragmentId = match.params.version
      const { agree, invitationId } = parseSearchParams(location.search)
      if (agree === 'true') {
        replace(location.pathname)
        reviewerDecision(invitationId, collectionId, fragmentId, true)
          .then(() => {
            getCollection({ id: collectionId })
            getFragment({ id: collectionId }, { id: fragmentId })
          })
          .catch(redirectToError(replace))
      }

      apiGet(`/users?editorInChief=true`).then(res =>
        setEditorInChief(head(res.users)),
      )
    },
  }),
  withProps(({ fragment }) => ({
    hasResponseToReviewers:
      !isEmpty(get(fragment, 'files.responseToReviewers')) ||
      get(fragment, 'commentsToReviewers'),
  })),
)(ManuscriptLayout)
